/*
 * Copyright 1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * 
 * UNPUBLISHED -- Rights reserved under the copyright laws of the United
 * States.   Use of a copyright notice is precautionary only and does not
 * imply publication or disclosure.
 * 
 * U.S. GOVERNMENT RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in FAR 52.227.19(c)(2) or subparagraph (c)(1)(ii) of the Rights
 * in Technical Data and Computer Software clause at DFARS 252.227-7013 and/or
 * in similar or successor clauses in the FAR, or the DOD or NASA FAR
 * Supplement.  Contractor/manufacturer is Silicon Graphics, Inc.,
 * 2011 N. Shoreline Blvd. Mountain View, CA 94039-7311.
 * 
 * THE CONTENT OF THIS WORK CONTAINS CONFIDENTIAL AND PROPRIETARY
 * INFORMATION OF SILICON GRAPHICS, INC. ANY DUPLICATION, MODIFICATION,
 * DISTRIBUTION, OR DISCLOSURE IN ANY FORM, IN WHOLE, OR IN PART, IS STRICTLY
 * PROHIBITED WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SILICON
 * GRAPHICS, INC.
 */

/*
 * app2.c
 *
 * Sample program to demonstrate use of the PCP trace performance metrics
 * domain agent (PMDA(3)).  This agent needs to be installed before metrics
 * can be made available via the performance metrics namespace (PMNS(4)),
 * and the Performance Metrics Collector Daemon (PMCD(1)).
 *
 * Once this program is running, the trace PMDA metrics & instances can be
 * viewed through PCP monitor tools such as pmchart(1), pmgadgets(1), and
 * pmview(1).  To view the help text associated with each of these metrics,
 * use:
 *   $ pminfo -tT trace
 */

#ident "$Id: app2.c,v 1.10 1997/09/05 00:15:15 nathans Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <pcp/trace.h>


#define IO_UPPER_LIMIT		1000	/* I/O ops */
#define CPU_UPPER_LIMIT		0xffff	/* iterations */
#define TIME_UPPER_LIMIT	10	/* seconds */

static void io_sucker(void);
static void cpu_sucker(void);
static void time_sucker(void);
static char *prog;

int
main(int argc, char **argv)
{
    int	i, sts;

    prog = argv[0];
    srand48(time(0));
    /* uncomment this for debugging information */
    /* pmtracestate(PMTRACE_STATE_API|PMTRACE_STATE_COMMS|PMTRACE_STATE_PDU); */
    /* uncomment this to use the asynchronous protocol */
    /* pmtracestate(PMTRACE_STATE_ASYNC); */

    for (i = 0;; i++) {
	if ((sts = pmtracepoint("mainloop")) < 0) {
	    fprintf(stderr, "%s: mainloop point trace failed (%d): %s\n",
						prog, sts, pmtraceerrstr(sts));
	    exit(1);
	}
	switch(i % 3) {
	case 0:
	    time_sucker();
	    break;
	case 1:
	    io_sucker();
	    break;
	case 2:
	    cpu_sucker();
	    break;
	}
    }
}


static void
cpu_sucker(void)
{
    int		i, j, sts;
    double	array[100];
    long	iterations;

    if ((sts = pmtracebegin("cpu_sucker")) < 0) {
	fprintf(stderr, "%s: cpu_sucker begin (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }

    iterations = lrand48() % CPU_UPPER_LIMIT;
    memset((void *)array, 0, 100*sizeof(double));

    for (i = 0; i < iterations; i++)
	for (j = 0; j < 100; j++)
	    array[j] = (double)(j*iterations);

    if ((sts = pmtraceend("cpu_sucker")) < 0) {
	fprintf(stderr, "%s: cpu_sucker end (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }
}

static void
time_sucker(void)
{
    long	seconds;
    int		sts;

    if ((sts = pmtracebegin("time_sucker")) < 0) {
	fprintf(stderr, "%s: time_sucker start (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }

    seconds = lrand48() % TIME_UPPER_LIMIT;
    sleep((unsigned int)seconds);

    if ((sts = pmtraceend("time_sucker")) < 0) {
	fprintf(stderr, "%s: time_sucker end (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }
}

static void
io_sucker(void)
{
    long	characters;
    FILE	*foo;
    int		i, sts;

    if ((sts = pmtracebegin("io_sucker")) < 0) {
	fprintf(stderr, "%s: io_sucker start (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }

    if ((foo = fopen("/dev/null", "rw")) == NULL) {
	fprintf(stderr, "%s: io_sucker can't open /dev/null.\n", prog);
	return;
    }

    characters = lrand48() % IO_UPPER_LIMIT;
    for (i = 0; i < characters; i++) {
	fgetc(foo);
	fputc('!', foo);
    }

    if ((sts = pmtraceend("io_sucker")) < 0) {
	fprintf(stderr, "%s: io_sucker end (%d): %s\n",
		prog, sts, pmtraceerrstr(sts));
	return;
    }
}
